CompletionWindow = class(Turbine.UI.Lotro.Window);
CompletionWindow.instance = nil;

-- Use a single window.
-- If the user closes it, clear the currently shown deeds.
-- If a deed is added while it's closed, open it.
-- If a deed is added while it's open, add it to the list.
-- If the last deed is acknowledged, close the window.

function CompletionWindow.GetInstance()
    if (CompletionWindow.instance ~= nil) then
        return CompletionWindow.instance;
    end
    return CompletionWindow();
end

function CompletionWindow:Constructor()
    -- Enforce only one such window:
    if (CompletionWindow.instance) then
        return;
    end
    Turbine.UI.Lotro.Window.Constructor(self);
    CompletionWindow.instance = self;

    self.deedHeight = 50;
    self.maxDeedAreaHeight = 400;
    self.loading = false;

    self:CreateWindow();
end

function CompletionWindow:CreateWindow()
    -- Main window to hold all the elements
    self:SetMinimumSize(550, 200);
    self:SetMaximumSize(550, 600);
    self:SetSize(SETTINGS.COMPLETEWIN.WIDTH,SETTINGS.COMPLETEWIN.HEIGHT);
    self:SetPosition(SETTINGS.COMPLETEWIN.X,SETTINGS.COMPLETEWIN.Y);
    self:SetResizable(true);
    Onscreen(self); -- Makes sure the window is still onscreen (perhaps user changed resolution since last playing)

    self.allDeedsScrollBar = Turbine.UI.Lotro.ScrollBar();
    self.allDeedsScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.allDeedsScrollBar:SetParent(self);

    self.allDeedsControl = Turbine.UI.ListBox();
    self.allDeedsControl:SetParent(self);
    self.allDeedsControl:SetPosition(20, 50);
    self.allDeedsControl:SetVerticalScrollBar(self.allDeedsScrollBar);
    
    --self.allDeedsControl:SetSize(425, 200);
    --self.allDeedsControl:SetBackColor(Turbine.UI.Color.Gray);

    -- Print to Console button:
    self.printToConsoleButton = Turbine.UI.Lotro.Button();
    self.printToConsoleButton:SetParent(self);
    self.printToConsoleButton:SetText(GetString(_LANG.COMPLETE_WIN.LOG_TO_CONSOLE_BUTTON));
    self.printToConsoleButton:SetSize(75, 25);
    self.printToConsoleButton.Click = function(sender, args)
        local mainWin = DeedTrackerWin.GetInstance();
        local uiCharacter = mainWin:GetUiCharacter();

        local text = GetString(_LANG.COMPLETE_WIN.LOG_TO_CONSOLE_HEADER) .. MYCHAR:GetName() .. ":";
        for index=self.allDeedsControl:GetItemCount(),1,-1 do
            local i, j, k = unpack(GetIJKFromIndicies(self.allDeedsControl:GetItem(index).Indicies));
            local saveIndex = GetDeedSaveIndex(i, j, k);

            local saveEntry = _CHARDATA[uiCharacter]["DEEDS"][i][j][saveIndex];
            local completionTimestamp = "";
            if (saveEntry ~= nil) then
                completionTimestamp = saveEntry["WHEN"] .. ": "
            end

            text = text .. "\n" .. completionTimestamp .. GetFullyQualifiedDeedName(i, j, k);
        end
        Debug(text);
    end
    self.printToConsoleButton:SetPosition(self:GetWidth() - self.printToConsoleButton:GetWidth() - 50, 450);
    RegisterForStringTooltip(self.printToConsoleButton, _LANG.COMPLETE_WIN.LOG_TO_CONSOLE_BUTTON_TOOLTIP);

    -- Close button
    self.clearListButton = Turbine.UI.Lotro.Button();
    self.clearListButton:SetParent(self);
    self.clearListButton:SetText(GetString(_LANG.COMPLETE_WIN.WINDOW_BUTTON_CLEAR_LIST));
    self.clearListButton:SetSize(100, 25);
    self.clearListButton:SetPosition(self:GetWidth() / 2 - self.clearListButton:GetWidth() / 2, 450);
    self.clearListButton.Click = function(sender,args)
        self:Hide();
        self.deedHeight = 50;
        for index=1, self.allDeedsControl:GetItemCount() do
            local i, j, k = unpack(GetIJKFromIndicies(self.allDeedsControl:GetItem(index).Indicies));
            DeedTrackerWin.GetInstance():ClearMainWindowDeed(i, j, k);
        end
        self.allDeedsControl:ClearItems();
        self:PostAddOrRemove();
    end
    RegisterForStringTooltip(self.clearListButton, _LANG.COMPLETE_WIN.WINDOW_BUTTON_CLEAR_LIST_TOOLTIP);


    self.PositionChanged = function()
        SETTINGS.COMPLETEWIN.X = self:GetLeft();
        SETTINGS.COMPLETEWIN.Y = self:GetTop();
    end

    self.SizeChanged = function(sender, args)
        SETTINGS.COMPLETEWIN.WIDTH = self:GetWidth();
        SETTINGS.COMPLETEWIN.HEIGHT = self:GetHeight();
        self:RepositionElementsAfterResize();
    end

    self.Closed = function(sender, args)
        --Debug("Completion Window Closed");
        -- Remove "new" color from existing entries.
        for i=1, self.allDeedsControl:GetItemCount() do
            self.allDeedsControl:GetItem(i):SetBackColor(Turbine.UI.Color.Black)
        end
        self:Hide();
    end


    self:RepositionElementsAfterResize();
end

function CompletionWindow:RepositionElementsAfterResize()
    local width = self:GetWidth();
    local height = self:GetHeight();

    self.allDeedsControl:SetSize(width-48,(height-100));
    self.allDeedsScrollBar:SetPosition(self.allDeedsControl:GetLeft()+self.allDeedsControl:GetWidth()+2,self.allDeedsControl:GetTop());
    self.allDeedsScrollBar:SetSize(8,self.allDeedsControl:GetHeight());
    self.clearListButton:SetPosition(self:GetWidth() / 2 - self.clearListButton:GetWidth() / 2, self:GetHeight() - 40);
    self.printToConsoleButton:SetPosition(self:GetWidth() - self.printToConsoleButton:GetWidth() - 50, self:GetHeight() - 40);
end

function CompletionWindow:AdjustTotalHeight()

    if (self.deedHeight > self.maxDeedAreaHeight) then
        self.allDeedsControl:SetHeight(self.maxDeedAreaHeight);
    else
        self.allDeedsControl:SetHeight(self.deedHeight);
    end
    self:SetHeight(self.allDeedsControl:GetHeight() + 100);
    self:RepositionElementsAfterResize();
end

function CompletionWindow:PostAddOrRemove()
    self:UpdateWindowTitle();
    self:AdjustTotalHeight();

    -- Disable the Main Window button if there are none left:
    DeedTrackerWin.GetInstance():SetCompletedWindowButtonEnabled(self.allDeedsControl:GetItemCount() > 0);
end

function CompletionWindow:UpdateWindowTitle()
    local count = self.allDeedsControl:GetItemCount();
    local title = "Deed Tracker - " .. GetString(_LANG.COMPLETE_WIN.WINDOW_TITLE) .. " (" .. count .. ")";

    self:SetText(title);
end

function CompletionWindow:AddDeed(i, j, k)
    local mainWin = DeedTrackerWin.GetInstance();

    if (_DEED_LOG_PAGE_TAB_CONTENTS[i][j][k] == nil) then
        return;
    end

    if (not self.loading) then
        self:Show();
    end

    local deedControl = GetDeedInformationControl(i, j, k, self:GetWidth(), false);
    local y = deedControl:GetHeight();

    if (not self.loading) then
        deedControl:SetBackColor(NEW_DEED_COLOR);
    end
    self.allDeedsControl:InsertItem(0, deedControl);

    -- Button to go to place in deed tracker:
    local gotoButton = Turbine.UI.Lotro.GoldButton();
    gotoButton:SetParent(deedControl);
    gotoButton:SetText(GetString(_LANG.COMPLETE_WIN.WINDOW_DEED_GOTO));
    gotoButton:SetSize(125, 25);
    gotoButton:SetTop(y);
    gotoButton.Click = function(sender,args)
        mainWin:ShowMainWindow();
        mainWin:ShowMainWindowDeed(i, j, k);
    end
    RegisterForStringTooltip(gotoButton, _LANG.COMPLETE_WIN.WINDOW_DEED_GOTO_TOOLTIP);

    -- Button to clear this item only:
    local clearButton = Turbine.UI.Lotro.Button();
    clearButton:SetParent(deedControl);
    clearButton:SetText(GetString(_LANG.COMPLETE_WIN.WINDOW_BUTTON_CLEAR_ITEM));
    clearButton:SetSize(25, 25);
    clearButton:SetTop(y);
    clearButton.Click = function(sender,args)
        self.allDeedsControl:RemoveItem(deedControl);
        mainWin:ClearMainWindowDeed(i, j, k);
        self:PostAddOrRemove();
    end
    RegisterForStringTooltip(clearButton, _LANG.COMPLETE_WIN.WINDOW_BUTTON_CLEAR_ITEM_TOOLTIP);

    y = y + gotoButton:GetHeight();

    if (self.allDeedsControl:GetItemCount() > 1) then
        -- add a spacer between two deeds:
        local lineBetween = Turbine.UI.Control();
        lineBetween:SetParent(deedControl);
        lineBetween:SetBackColor(Turbine.UI.Color.Gray);
        lineBetween:SetSize(self:GetWidth() - 80, 1);
        lineBetween:SetPosition(0, y + 5);
        y = y + 15;
    end


    deedControl:SetSize(self:GetWidth() - 50, y);
    deedControl:SetPosition(20, self.deedHeight);
    gotoButton:SetLeft(deedControl:GetWidth() / 2 - gotoButton:GetWidth() / 2);
    clearButton:SetLeft(deedControl:GetWidth() - 50);
    self.deedHeight = self.deedHeight + deedControl:GetHeight() + 10;
    self:AdjustTotalHeight();

    self:PostAddOrRemove();
end

function CompletionWindow:Show()
    if (not SETTINGS.DO_NOT_SHOW_COMPLETION_WINDOW) then
        SETTINGS.COMPLETEWIN.VISIBLE = true;
        self:SetVisible(true);
        self:Activate();
    end
end

function CompletionWindow:Hide()
    SETTINGS.COMPLETEWIN.VISIBLE = false;
    self:SetVisible(false);
end

function CompletionWindow:SaveUnclearedDeeds(chardata)
    chardata["UNCLEARED"] = {};
    for index=1, self.allDeedsControl:GetItemCount() do
        chardata.UNCLEARED[index] = self.allDeedsControl:GetItem(index).Indicies;
    end
end

function CompletionWindow:LoadUnclearedDeeds(chardata)
    if (chardata["UNCLEARED"] == nil) then return; end

    local count = table.maxn(chardata["UNCLEARED"]);

    self.loading = true;
    for index=count, 1, -1 do
        local i, j, k = unpack(GetIJKFromIndicies(chardata["UNCLEARED"][index]));
        self:AddDeed(i, j, k);
    end
    self.loading = false;

    if (SETTINGS.COMPLETEWIN.VISIBLE) then
        self:Show();
    end
end
