-- Turbine Imports..
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

-- Plugin Imports..
import "CubePlugins.DeedTracker.Globals";
import "CubePlugins.DeedTracker.PluginFunctions";
import "CubePlugins.DeedTracker.ChatLogger";
import "CubePlugins.DeedTracker.Commands";
import "CubePlugins.DeedTracker.GeneralFunctions";
import "CubePlugins.DeedTracker.Strings";
import "CubePlugins.DeedTracker.Onscreen";
import "CubePlugins.DeedTracker.VindarPatch";
import "CubePlugins.DeedTracker.DropDown";
import "CubePlugins.DeedTracker.Images";
import "CubePlugins.DeedTracker.DeedInfo";
import "CubePlugins.DeedTracker.DeedObjectiveInfo";
import "CubePlugins.DeedTracker.RegionInfo";
import "CubePlugins.DeedTracker.ConflictingQuests";
import "CubePlugins.DeedTracker.DeedItems";
import "CubePlugins.DeedTracker.Timer";

-- Windows --
import "CubePlugins.DeedTracker.Options";
import "CubePlugins.DeedTracker.MainWin";
import "CubePlugins.DeedTracker.DeedTooltipWindow";
import "CubePlugins.DeedTracker.StringTooltipWindow";
import "CubePlugins.DeedTracker.DisambiguateWindow";
import "CubePlugins.DeedTracker.DeedOrQuestWindow";
import "CubePlugins.DeedTracker.CompletionWindow";
import "CubePlugins.DeedTracker.MinimizedIcon";

function UnloadReloader()
    Turbine.PluginManager.RefreshAvailablePlugins();
    local loadedPlugins = Turbine.PluginManager.GetLoadedPlugins();

    reloadedUnloader = Turbine.UI.Control();
    reloadedUnloader:SetWantsUpdates( true );
    
    reloadedUnloader.Update = function( sender, args )
        Turbine.PluginManager.RefreshAvailablePlugins();
        local loadedPlugins = Turbine.PluginManager.GetLoadedPlugins();

        for k,v in pairs(loadedPlugins) do
            if v.Name == "Deed Tracker Reloader" then
                Turbine.PluginManager.UnloadScriptState( 'DeedTrackerReloader' );
            end
        end
        reloadedUnloader:SetWantsUpdates( false );
    end
end

function saveData()
    local mainWin = DeedTrackerWin.GetInstance();
    mainWin:SaveRegionExpandedStates();

    local uiCharacter = mainWin:GetUiCharacter();
    CompletionWindow.GetInstance():SaveUnclearedDeeds(_CHARDATA[uiCharacter]);
    PatchDataSave(Turbine.DataScope.Character, "DeedTracker_Settings", SETTINGS);
    PatchDataSave(Turbine.DataScope.Server, "DeedTracker_CharData", _CHARDATA);
end

function loadData()
    local SavedSettings = {};
    SavedSettings = PatchDataLoad(Turbine.DataScope.Character, "DeedTracker_Settings");

    -- Check the saved settings to make sure it is still compatible with newer updates, add in any missing default settings
    if type(SavedSettings) == 'table' then
        local tempSETTINGS = {};
        tempSETTINGS = deepcopy(DEFAULT_SETTINGS);
        SETTINGS = mergeTables(tempSETTINGS,SavedSettings);
    else
        SETTINGS = deepcopy(DEFAULT_SETTINGS);
    end

    local SavedCharData = {};
    SavedCharData = PatchDataLoad(Turbine.DataScope.Server, "DeedTracker_CharData");

    -- Check the saved settings to make sure it is still compatible with newer updates, add in any missing default settings
    if type(SavedCharData) == 'table' then
        _CHARDATA = deepcopy(SavedCharData);
    end

end

function updateData()
    -- v1.1.0:
    -- Realized we shouldn't store possible character names in _CHARDATA, so changed "SEARCH" and "SERVER" to ":SEARCH" and ":SERVER".
    -- Convert existing ones, to make like easier for testers, but remove eventually in case someone *does* have these names.
    if (_CHARDATA["SEARCH"] ~= nil) then
        _CHARDATA[":SEARCH"] = _CHARDATA["SEARCH"];
        _CHARDATA["SEARCH"] = nil;
    end

    if (_CHARDATA["SERVER"] ~= nil) then
        if (_CHARDATA["SERVER"]["DEEDS"] ~= nil) then
            _CHARDATA["SERVER"]["DEEDS"] = nil;
        end
        _CHARDATA[":SERVER"] = _CHARDATA["SERVER"];
        _CHARDATA["SERVER"] = nil;
    end

end

function DrawWindows()
     DeedTrackerWin.GetInstance():DrawMainWin();
end

Turbine.Plugin.Unload = function (sender, args)
    -- Save the data when the plugin unloads.
    saveData();
    Debug(GetString(_LANG.STATUS.UNLOADED));
end

function SetServerLevelCap()
    local isLegendaryServer = LoadServerField("LEGENDARY_SERVER");

    local legendaryServerLevelCap = tonumber(LoadServerField("LEGENDARY_SERVER_LEVEL_CAP"));
    local serverLevelCap = tonumber(LoadServerField("SERVER_LEVEL_CAP"));

    if (LEGENDARY_SERVER_LEVEL_CAP > legendaryServerLevelCap) then
        if (SETTINGS.VERBOSE_OUTPUT) then
            Debug("DeedTracker: Updating Legendary Server level cap to " .. LEGENDARY_SERVER_LEVEL_CAP);
        end

        legendaryServerLevelCap = LEGENDARY_SERVER_LEVEL_CAP;
        SaveServerField("LEGENDARY_SERVER_LEVEL_CAP", LEGENDARY_SERVER_LEVEL_CAP);
    end

    if (SERVER_LEVEL_CAP > serverLevelCap) then
        if (SETTINGS.VERBOSE_OUTPUT) then
            Debug("DeedTracker: Updating Server level cap to " .. SERVER_LEVEL_CAP);
        end

        serverLevelCap = SERVER_LEVEL_CAP;
        SaveServerField("SERVER_LEVEL_CAP", SERVER_LEVEL_CAP);
    end

    if (isLegendaryServer) then
        CURRENT_LEVEL_CAP = legendaryServerLevelCap;
    else
        CURRENT_LEVEL_CAP = serverLevelCap;
    end
end

function Main()
    -- Make sure this isn't session play.
    local name = MYCHAR:GetName();
    if (name:sub(1,1) == "~") then
        Debug(GetString(_LANG.ERRORS.SESSION_PLAY));
        return;
    end

    -- Initiate load sequence
    UnloadReloader();
    loadData();
    updateData();
    CheckCharData();
    SetServerLevelCap();
    DrawWindows();
    InitiateChatLogger();
    RegisterCommands();
    CreateOptionsContent();

    -- Load text
    Debug(GetString(_LANG.STATUS.LOADED));

    -- Check if session play completed a deed by delivering an item:
    CheckForDeedItems();
end

Main();
